@@ -4,7 +4,6 @@ from enum import Enum
 from typing import (
     Any,
     Callable,
-    Iterable,
     Iterator,
     Literal,
     MutableSequence,
@@ -13,11 +12,11 @@ from typing import (
     TypeVar,
     overload,
 )
-from dataclasses import dataclass
 from functools import wraps
 
 from ._reversible import ReversibleFunction
 from ._undoable import UndoableInterface, UndoableProperty
+from ._command import Command, _CommandBase
 from ._const import empty
 
 if TYPE_CHECKING:
@@ -26,109 +25,6 @@ if TYPE_CHECKING:
 _F = TypeVar("_F", bound=Callable)
 
 
-def _fmt_arg(v: Any) -> str:
-    v_repr = repr(v)
-    if len(v_repr) > 14:
-        v_repr = "#" + type(v).__name__ + "#"
-    return v_repr
-
-
-@dataclass(repr=False)
-class Command:
-    """
-    Undoable command object.
-
-    A command is a function call with arguments and keyword arguments.
-
-    Parameters
-    ----------
-    func : callable
-        The reversible function.
-    args : tuple
-        The positional arguments.
-    kwargs : dict
-        The keyword arguments.
-    size : float
-        The size of the command. This value is used to determine when older commands
-        should be removed from the stack.
-    """
-
-    func: ReversibleFunction
-    args: tuple[Any, ...]
-    kwargs: dict[str, Any]
-    size: float = 0.0
-
-    def __repr__(self) -> str:
-        _cls = type(self).__name__
-        _args = list(map(_fmt_arg, self.args))
-        _args += list(f"{k}={_fmt_arg(v)}" for k, v in self.kwargs.items())
-        _args = ", ".join(_args)
-        _fn = self.func.__name__
-        return f"{_cls}<{_fn}({_args})>"
-
-    def _call_with_callback(self):
-        return self.func._call_with_callback(*self.args, **self.kwargs)
-
-    def _call_raw(self):
-        return self.func._call_raw(*self.args, **self.kwargs)
-
-    def _revert(self):
-        return self.func._revert(*self.args, **self.kwargs)
-
-    @classmethod
-    def merge(cls, cmds: Iterable[Command], name: str | None = None) -> Command:
-        """
-        Merge multiple commands into one.
-
-        This method is used to reduce command set stack by merging simple operations.
-        For instance, if you defined a undoable "append" method and intend to define
-        "extend" method by repeating "append" operation, a lot of "append" command
-        will be generated. To avoid this, you can use this method to merge "append".
-
-        Parameters
-        ----------
-        cmds : Iterable[CommandSet]
-            List of command sets to merge.
-
-        Returns
-        -------
-        CommandSet
-            Merged command set.
-        """
-        arguments: list[tuple, dict[str, Any]] = []
-        commands: list[ReversibleFunction] = []
-        total_size = 0.0
-
-        for cmd in cmds:
-            commands.append(cmd.func)
-            arguments.append((cmd.args, cmd.kwargs))
-            total_size += cmd.size
-
-        fn_merged = ReversibleFunction.merge(commands)
-        if name is not None:
-            fn_merged.__name__ = name
-        return cls(fn_merged, (arguments,), {}, size=total_size)
-
-    def to_code(self, ns: str | None = None) -> str:
-        """
-        Convert command to code.
-
-        Parameters
-        ----------
-        ns : str, optional
-            The namespace of the command. If None, the command will be converted
-            to a simple expression.
-
-        Returns
-        -------
-        str
-            The code of the command.
-        """
-        from ._codegen import generate
-
-        return generate(self, ns)
-
-
 class CallbackList(MutableSequence[_F]):
     """
     A list of callbacks of UndoManager updates.
@@ -206,15 +102,15 @@ class ManagerState:
         self.measure = measure
         self.maxsize = maxsize
         self.is_blocked = False
-        self.stack_undo: list[Command] = []
-        self.stack_redo: list[Command] = []
+        self.stack_undo: list[_CommandBase] = []
+        self.stack_redo: list[_CommandBase] = []
         self.stack_undo_size = 0.0
         self.stack_redo_size = 0.0
         self.called_callbacks: CallbackList[
-            Callable[[Command, CallType], Any]
+            Callable[[_CommandBase, CallType], Any]
         ] = CallbackList()
         self.errored_callbacks: CallbackList[
-            Callable[[Command, Exception], Any]
+            Callable[[_CommandBase, Exception], Any]
         ] = CallbackList()
 
 
@@ -244,13 +140,12 @@ class UndoManager:
             s_redo = "[]"
         return f"{cls_name}(\n  undo={s_undo},\n  redo={s_redo}\n)"
 
-    def __get__(self, obj, objtype=None) -> UndoManager:
+    def __get__(self, obj, objtype=None) -> Self:
         if obj is None:
             return self
         _id = id(obj)
         if (stack := self._instances.get(_id, None)) is None:
-            stack = type(self)()
-            self._instances[_id] = stack
+            self._instances[_id] = stack = type(self)()
         return stack
 
     @property
@@ -259,12 +154,12 @@ class UndoManager:
         return self._state.is_blocked
 
     @property
-    def called(self) -> CallbackList[Callable[[Command, CallType], Any]]:
+    def called(self) -> CallbackList[Callable[[_CommandBase, CallType], Any]]:
         """Callback list for called events."""
         return self._state.called_callbacks
 
     @property
-    def errored(self) -> CallbackList[Callable[[Command, Exception], Any]]:
+    def errored(self) -> CallbackList[Callable[[_CommandBase, Exception], Any]]:
         """Callback list for errored events."""
         return self._state.errored_callbacks
 
@@ -296,7 +191,7 @@ class UndoManager:
         self.called.evoke(cmd, CallType.redo)
         return out
 
-    def link(self, other: UndoManager) -> None:
+    def link(self, other: Self) -> None:
         if not isinstance(other, UndoManager):
             raise TypeError(f"Cannot link {other!r}.")
         if self.empty:
@@ -316,12 +211,12 @@ class UndoManager:
     #     return out
 
     @property
-    def stack_undo(self) -> list[Command]:
+    def stack_undo(self) -> list[_CommandBase]:
         """List of undo stack."""
         return list(self._state.stack_undo)
 
     @property
-    def stack_redo(self) -> list[Command]:
+    def stack_redo(self) -> list[_CommandBase]:
         """List of redo stack."""
         return list(self._state.stack_redo)
 
@@ -343,7 +238,7 @@ class UndoManager:
         """True if stack is empty."""
         return len(self._state.stack_undo) == 0 and len(self._state.stack_redo) == 0
 
-    def append(self, cmd: Command) -> None:
+    def append(self, cmd: _CommandBase) -> None:
         """Append new command to the undo stack."""
         if self.is_blocked:
             return None
@@ -456,20 +351,22 @@ class UndoManager:
 
         return _undef
 
-    def merge_commands(self, start: int, stop: int, name: str | None = None) -> None:
+    def merge_commands(
+        self, start: int, stop: int, formatter: Callable | None = None
+    ) -> None:
         """Merge a command set into the undo stack."""
-        merged = Command.merge(self._state.stack_undo[start:stop], name=name)
+        merged = Command.merge(self._state.stack_undo[start:stop], formatter=formatter)
         del self._state.stack_undo[start:stop]
         self._state.stack_undo.insert(start, merged)
         return None
 
     @contextmanager
-    def merging(self, name: str | None = None) -> None:
+    def merging(self, formatter: Callable | None = None) -> None:
         """Merge all the commands into a single command in this context."""
         len_before = len(self._state.stack_undo)
         yield None
         len_after = len(self._state.stack_undo)
-        self.merge_commands(len_before, len_after, name=name)
+        self.merge_commands(len_before, len_after, formatter=formatter)
         return None
 
     @contextmanager
